@@ -46,18 +46,98 @@ class Integrations(ApiWrapper):
     def __init__(self, parent):
         super(Integrations, self).__init__(parent.api, 'integrations')
 
-    def types(self):
+    def itypes(self):
         return Types(self)
 
     def instances(self):
         return Instances(self)
 
-    def create_instance(self, type_name, definition):
-        resp = self.api.post('install/%s' % type_name, json=definition)
+    def available(self):
+        # Compatibility function because this is the name that the
+        # OpsRamp API docs use for this data set.
+        return self.itypes()
+
+    def installed(self):
+        # Compatibility function because this is the name that the
+        # OpsRamp API docs use for this data set.
+        return self.instances()
+
+
+class Types(ApiWrapper):
+    def __init__(self, parent):
+        super(Types, self).__init__(parent.api, 'available')
+
+    def search(self, pattern=''):
+        suffix = '/search'
+        if pattern:
+            suffix += '?' + pattern
+        return self.api.get(suffix)
+
+
+class Instances(ApiWrapper):
+    def __init__(self, parent):
+        super(Instances, self).__init__(parent.api, 'installed')
+        # The OpsRamp "create instance" API endpoint is in an
+        # atypical place in the tree so need to compute it.
+        self.creator_api = parent.api.clone()
+        self.creator_api.chroot('install')
+
+    def search(self, pattern=''):
+        suffix = '/search'
+        if pattern:
+            suffix += '?' + pattern
+        return self.api.get(suffix)
+
+    def create(self, type_name, definition):
+        resp = self.creator_api.post(type_name, json=definition)
         return resp
 
-    def instance(self, uuid):
-        return SingleInstance(self, uuid)
+    def update(self, uuid, definition):
+        return self.api.post('%s' % uuid, json=definition)
+
+    def set_auth_type(self, uuid, auth_type):
+        assert auth_type in ('OAUTH2', 'WEBHOOK', 'BASIC')
+        settings = {
+            'authType': auth_type
+        }
+        return self.api.post('%s/inbound/authentication' % uuid, json=settings)
+
+    def enable(self, uuid):
+        return self.api.post('%s/enable' % uuid)
+
+    def disable(self, uuid):
+        return self.api.post('%s/disable' % uuid)
+
+    def notifier(self, uuid, definition):
+        return self.api.post('%s/notifier' % uuid, json=definition)
+
+    # A helper function that extracts the authentication types from typical
+    # integration instance response structs.
+    @staticmethod
+    def auth_type(resp):
+        in_auth = resp.get(
+            'inboundConfig', {}).get(
+                'authentication', {}).get(
+                    'authType', None)
+        out_auth = resp.get(
+            'outboundConfig', {}).get(
+                'authentication', {}).get(
+                    'authType', None)
+        return (in_auth, out_auth)
+
+    # A helper function that looks for login credentials in typical
+    # integration instance response structs and redacts them.
+    @staticmethod
+    def redact_response(resp):
+        for key in ('inboundConfig', 'outboundConfig'):
+            config = resp.get(key, {})
+            auth = config.get('authentication', {})
+            if 'token' in auth:
+                auth['token'] = 'REDACTED'
+            for creds in auth.get('apiKeyPairs', {}):
+                for i in creds.keys():
+                    creds[i] = 'REDACTED'
+        return resp
 
     # Helper functions to create the complex structures that OpsRamp
     # uses to define new integration instances. There are lots of
@@ -82,7 +162,7 @@ class Integrations(ApiWrapper):
     def mkEmailAlert(display_name,
                      logo_fname=None):
         assert display_name
-        retval = Integrations.mkBase(display_name, logo_fname)
+        retval = Instances.mkBase(display_name, logo_fname)
         return retval
 
     @staticmethod
@@ -91,7 +171,7 @@ class Integrations(ApiWrapper):
                  parent_uuid=None,
                  inbound_auth_type=None):
         assert display_name
-        retval = Integrations.mkBase(display_name, logo_fname)
+        retval = Instances.mkBase(display_name, logo_fname)
         if parent_uuid:
             retval['parentIntg'] = {
                 'id': parent_uuid
@@ -143,60 +223,47 @@ class Integrations(ApiWrapper):
         }
         return retval
 
-    def available(self):
-        # Compatibility function because this is the name that the
-        # OpsRamp API docs use for this data set.
-        return self.types()
-
-    def installed(self):
-        # Compatibility function because this is the name that the
-        # OpsRamp API docs use for this data set.
-        return self.instances()
-
-
-class Types(ApiWrapper):
-    def __init__(self, parent):
-        super(Types, self).__init__(parent.api, 'available')
-
-    def search(self, pattern=''):
-        suffix = '/search'
-        if pattern:
-            suffix += '?' + pattern
-        return self.api.get(suffix)
-
-
-class Instances(ApiWrapper):
-    def __init__(self, parent):
-        super(Instances, self).__init__(parent.api, 'installed')
-
-    def search(self, pattern=''):
-        suffix = '/search'
-        if pattern:
-            suffix += '?' + pattern
-        return self.api.get(suffix)
-
-
-class SingleInstance(ApiWrapper):
-    def __init__(self, parent, uuid):
-        api = parent.api.clone()
-        api.chroot('/installed')
-        super(SingleInstance, self).__init__(api, uuid)
-
-    def enable(self):
-        return self.api.post('/enable')
-
-    def disable(self):
-        return self.api.post('/disable')
-
-    def notifier(self, definition):
-        return self.api.post('/notifier', json=definition)
-
-    def update(self, definition):
-        return self.api.post(json=definition)
-
-    def set_auth_type(self, auth_type):
-        assert auth_type in ('OAUTH2', 'WEBHOOK', 'BASIC')
-        settings = {
+    @staticmethod
+    def mkBaseNotifier(api_type, base_uri,
+                       auth_type='OAUTH2',
+                       grant_type='CLIENT_CREDENTIALS',
+                       access_token_uri=None,
+                       api_key=None,
+                       api_secret=None,
+                       user_name=None,
+                       password=None):
+        assert api_type in ('REST_API', 'SOAP_API')
+        assert base_uri
+        assert auth_type in ('NONE', 'BASIC', 'OAUTH2')
+        retval = {
+            'type': api_type,
+            'baseURI': base_uri,
             'authType': auth_type
         }
-        return self.api.post('inbound/authentication', json=settings)
+        if auth_type != 'NONE':
+            retval['grantType'] = grant_type
+            if auth_type == 'OAUTH2':
+                assert grant_type in ('CLIENT_CREDENTIALS', 'PASSWORD')
+                # empty string is ok but omitting these isn't.
+                assert access_token_uri is not None
+                assert api_key is not None
+                assert api_secret is not None
+                retval['accessTokenURI'] = access_token_uri
+                retval['apiKey'] = api_key
+                retval['apiSecret'] = api_secret
+            if grant_type == 'PASSWORD':
+                # empty string is ok but omitting these isn't.
+                assert user_name is not None
+                assert password is not None
+                retval['userName'] = user_name
+                retval['password'] = password
+        return retval
+
+    # mkValueMap(('attrValue', 'tenantAttrValue'),
+    #            (('client_113', '22cdbc5bb401d737b088c9'),
+    #             ('client_843', '66cdbc5bb401d737b088c9'))
+    @staticmethod
+    def mkValueMap(labels, value_pairs):
+        ormpname, othername = labels
+        retval = [{ormpname: x[0], othername: x[1]} for x in value_pairs]
+        return retval
